@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 our $DEFAULT_UA = "Perl + " . __PACKAGE__ . "/$VERSION";
 our $DEFAULT_MAXREDIR = 3;
@@ -145,30 +145,25 @@ sub get_connection
    if( my $cr = $self->{connections}->{"$host:$port"} ) {
       my ( $conn ) = @$cr;
 
-      my $on_read = $on_ready->( $conn );
-      push @$cr, $on_read;
+      $on_ready->( $conn );
 
       return;
    }
 
-   if( $args{handle} ) {
-      my $on_read;
-
-      my $cr;
-
+   if( $args{transport} ) {
       my $conn = Net::Async::HTTP::Client->new(
-         handle => $args{handle},
+         transport => $args{transport},
 
          on_closed => sub {
             delete $self->{connections}->{"$host:$port"};
          },
       );
 
-      $cr = $self->{connections}->{"$host:$port"} = [ $conn ];
+      $self->{connections}->{"$host:$port"} = [ $conn ];
 
       $self->add_child( $conn );
 
-      $on_read = $on_ready->( $conn );
+      $on_ready->( $conn );
 
       return;
    }
@@ -188,7 +183,10 @@ sub get_connection
 
       on_connected => sub {
          my ( $sock ) = @_;
-         $self->get_connection( %args, handle => $sock );
+
+         my $transport = IO::Async::Stream->new( handle => $sock );
+
+         $self->get_connection( %args, transport => $transport );
       },
    );
 }
@@ -392,8 +390,10 @@ sub do_request
    $request->init_header( 'User-Agent' => $self->{user_agent} ) if length $self->{user_agent};
 
    if( my $handle = $args{handle} ) { # INTERNAL UNDOCUMENTED
+      my $transport = IO::Async::Stream->new( handle => $handle );
+
       $self->get_connection(
-         handle => $handle,
+         transport => $transport,
 
          # To make the connection cache logic happy
          host => "[[local_io_handle]]",
